@@ -83,9 +83,10 @@ class DownloadAttachmentResource(Resource):
                 ),
             )
         except Exception:
-            current_app.logger.error(
-                f"Attachment file was not found for: {attachment_file_id}"
-            )
+            if config.LOG_FILE_NOT_FOUND:
+                current_app.logger.error(
+                    f"Attachment file was not found for: {attachment_file_id}"
+                )
             abort(404)
 
 
@@ -189,7 +190,7 @@ class CommentTaskResource(Resource):
             links,
         ) = self.get_arguments()
 
-        user_service.check_task_access(task_id)
+        user_service.check_task_action_access(task_id)
         user_service.check_task_status_access(task_status_id)
         files = request.files
 
@@ -288,7 +289,8 @@ class AttachmentResource(Resource):
         user = persons_service.get_current_user()
         comment = tasks_service.get_comment(comment_id)
         if comment["person_id"] != user["id"]:
-            permissions.check_admin_permissions()
+            task = tasks_service.get_task(task_id)
+            user_service.check_manager_project_access(task["project_id"])
 
         deletion_service.remove_attachment_file_by_id(attachment_id)
         return "", 204
@@ -331,7 +333,8 @@ class AddAttachmentToCommentResource(Resource):
         user = persons_service.get_current_user()
         comment = tasks_service.get_comment(comment_id)
         if comment["person_id"] != user["id"]:
-            permissions.check_admin_permissions()
+            task = tasks_service.get_task(task_id)
+            user_service.check_manager_project_access(task["project_id"])
 
         files = request.files
         comment = comments_service.add_attachments_to_comment(comment, files)
@@ -490,7 +493,13 @@ class ReplyCommentResource(Resource, ArgsMixin):
             ]
         )
 
-        user_service.check_task_access(task_id)
+        comment = tasks_service.get_comment(comment_id)
+        current_user = persons_service.get_current_user()
+        if comment["person_id"] != current_user["id"]:
+            if permissions.has_client_permissions():
+                raise permissions.PermissionDenied()
+            user_service.check_task_action_access(task_id)
+
         return comments_service.reply_comment(comment_id, args["text"])
 
 
@@ -529,7 +538,6 @@ class DeleteReplyCommentResource(Resource):
             200:
                 description: Given comment reply deleted
         """
-        user_service.check_task_access(task_id)
         reply = comments_service.get_reply(comment_id, reply_id)
         current_user = persons_service.get_current_user()
         if reply["person_id"] != current_user["id"]:
