@@ -15,7 +15,7 @@ from zou.app.models.comment import (
     acknowledgements_table,
     mentions_table,
     department_mentions_table,
-    preview_link_table,
+    CommentPreviewLink,
 )
 from zou.app.models.department import Department
 from zou.app.models.entity import Entity, EntityLink
@@ -376,25 +376,17 @@ def _get_entity_task_query():
 
 
 def _convert_rows_to_detailed_tasks(rows, relations=False):
-    results = []
-    for entry in rows:
-        (
-            task_object,
-            project_name,
-            task_type_name,
-            task_status_name,
-            entity_type_name,
-            entity_name,
-        ) = entry
-
-        task = get_task(str(task_object.id), relations=relations)
-        task["project_name"] = project_name
-        task["task_type_name"] = task_type_name
-        task["task_status_name"] = task_status_name
-        task["entity_type_name"] = entity_type_name
-        task["entity_name"] = entity_name
-        results.append(task)
-    return results
+    return [
+        {
+            **task_object.serialize(relations=relations),
+            "project_name": project_name,
+            "task_type_name": task_type_name,
+            "task_status_name": task_status_name,
+            "entity_type_name": entity_type_name,
+            "entity_name": entity_name,
+        }
+        for task_object, project_name, task_type_name, task_status_name, entity_type_name, entity_name in rows
+    ]
 
 
 def get_task_types_for_shot(shot_id):
@@ -587,6 +579,7 @@ def get_comments(task_id, is_client=False, is_manager=False):
                 comment["text"] = ""
                 comment["attachment_files"] = []
                 comment["checklist"] = []
+                comment["replies"] = []
                 tmp_comments.append(comment)
             elif is_allowed:
                 tmp_comments.append(comment)
@@ -595,11 +588,13 @@ def get_comments(task_id, is_client=False, is_manager=False):
 
 
 def _prepare_query(task_id, is_client, is_manager):
+    Editor = aliased(Person, name="editor_id")
     query = (
         Comment.query.order_by(Comment.created_at.desc())
         .filter_by(object_id=task_id)
         .join(Person, Comment.person_id == Person.id)
         .join(TaskStatus, Comment.task_status_id == TaskStatus.id)
+        .join(Editor, Comment.editor_id == Editor.id, isouter=True)
         .add_columns(
             TaskStatus.name,
             TaskStatus.short_name,
@@ -607,6 +602,9 @@ def _prepare_query(task_id, is_client, is_manager):
             Person.first_name,
             Person.last_name,
             Person.has_avatar,
+            Editor.first_name,
+            Editor.last_name,
+            Editor.has_avatar,
         )
     )
     if not is_manager and not is_client:
@@ -626,6 +624,9 @@ def _run_task_comments_query(query):
             person_first_name,
             person_last_name,
             person_has_avatar,
+            editor_first_name,
+            editor_last_name,
+            editor_has_avatar,
         ) = result
 
         comment_dict = comment.serialize()
@@ -635,6 +636,13 @@ def _run_task_comments_query(query):
             "has_avatar": person_has_avatar,
             "id": str(comment.person_id),
         }
+        if comment.editor_id is not None:
+            comment_dict["editor"] = {
+                "first_name": editor_first_name,
+                "last_name": editor_last_name,
+                "has_avatar": editor_has_avatar,
+                "id": str(comment.editor_id),
+            }
         comment_dict["task_status"] = {
             "name": task_status_name,
             "short_name": task_status_short_name,
@@ -694,9 +702,9 @@ def _build_department_mention_map_for_comments(comment_ids):
 def _build_preview_map_for_comments(comment_ids, is_client=False):
     preview_map = {}
     query = (
-        PreviewFile.query.join(preview_link_table)
-        .filter(preview_link_table.c.comment.in_(comment_ids))
-        .add_columns(preview_link_table.c.comment)
+        PreviewFile.query.join(CommentPreviewLink)
+        .filter(CommentPreviewLink.comment.in_(comment_ids))
+        .add_columns(CommentPreviewLink.comment)
     )
     for preview, comment_id in query.all():
         comment_id = str(comment_id)
@@ -1148,7 +1156,7 @@ def get_last_comment_map(task_ids):
     task_comment_map = {}
     comments = (
         Comment.query.filter(Comment.object_id.in_(task_ids))
-        .join(Person)
+        .join(Person, Comment.person_id == Person.id)
         .filter(Person.role != "client")
         .order_by(Comment.object_id, Comment.created_at)
         .all()
@@ -1297,6 +1305,7 @@ def get_or_create_status(
     is_retake=False,
     is_feedback_request=False,
     is_default=False,
+    is_wip=False,
     for_concept=False,
     is_artist_allowed=True,
     is_client_allowed=True,
@@ -1328,6 +1337,7 @@ def get_or_create_status(
             for_concept=for_concept,
             is_artist_allowed=is_artist_allowed,
             is_client_allowed=is_client_allowed,
+            is_wip=is_wip,
         )
         events.emit("task-status:new", {"task_status_id": task_status.id})
     return task_status.serialize()
@@ -1784,7 +1794,7 @@ def reset_task_data(task_id):
             TaskStatus.is_retake,
             TaskStatus.is_feedback_request,
             TaskStatus.is_done,
-            TaskStatus.short_name,
+            TaskStatus.is_wip,
         )
         .all()
     )
@@ -1795,13 +1805,13 @@ def reset_task_data(task_id):
         task_status_is_retake,
         task_status_is_feedback_request,
         task_status_is_done,
-        task_status_short_name,
+        task_status_is_wip,
     ) in comments:
         if task_status_is_retake and not previous_is_retake:
             retake_count += 1
         previous_is_retake = task_status_is_retake
 
-        if task_status_short_name.lower() == "wip" and real_start_date is None:
+        if task_status_is_wip and real_start_date is None:
             real_start_date = comment.created_at
 
         if task_status_is_feedback_request:
@@ -1836,7 +1846,7 @@ def reset_task_data(task_id):
     return task.serialize(relations=True)
 
 
-def get_persons_tasks_dates():
+def get_persons_tasks_dates(project_id=None):
     """
     For schedule usages, for each active person, it returns the first start
     date of all tasks of assigned to this person and the last end date.
@@ -1851,6 +1861,10 @@ def get_persons_tasks_dates():
         .group_by(Person.id)
         .join(Task.assignees)
     )
+
+    if project_id is not None:
+        query = query.filter(Task.project_id == project_id)
+
     return [
         {
             "person_id": str(person_id),
@@ -1866,6 +1880,8 @@ def get_open_tasks(
     task_status_id=None,
     project_id=None,
     person_id=None,
+    studio_id=None,
+    department_id=None,
     start_date=None,
     due_date=None,
     priority=None,
@@ -1966,8 +1982,26 @@ def get_open_tasks(
             query = query.filter(Task.assignees == None)
             query_stats = query_stats.filter(Task.assignees == None)
         else:
-            query = query.filter(Task.assignees.any(id=person_id))
-            query_stats = query_stats.filter(Task.assignees.any(id=person_id))
+            query = query.filter(
+                Task.assignees.any(Person.id.in_(person_id.split(",")))
+            )
+            query_stats = query_stats.filter(
+                Task.assignees.any(Person.id.in_(person_id.split(",")))
+            )
+
+    if studio_id is not None:
+        query = query.filter(Task.assignees.any(studio_id=studio_id))
+        query_stats = query_stats.filter(
+            Task.assignees.any(studio_id=studio_id)
+        )
+
+    if department_id is not None:
+        query = query.filter(
+            Task.assignees.any(Person.departments.any(id=department_id))
+        )
+        query_stats = query_stats.filter(
+            Task.assignees.any(Person.departments.any(id=department_id))
+        )
 
     if start_date is not None:
         start_date = func.cast(start_date, Task.start_date.type)
